--[[
	/####################################\
	##                                  ##
	##      TaskMemorizer fr EEP       ##
	##                                  ##
	##   Autor: Benjamin Hogl (BH2),    ##
	##          benjamin.hogl@gmx.de    ##
	##   Version 0.2, 2014-12-08        ##
	##                                  ##
	\####################################/

 Steuerung zum Abarbeiten von wiederkehrenden Aufgaben, 
       je nach Anforderung durch Statussignale
]]

EEPGetElem = EEPGetElem or function(id)
	local signalRes=EEPGetSignal(id)
	if signalRes>0 then return signalRes
	else return EEPGetSwitch(id) end
end
EEPSetElem = EEPSetElem or function(id,pos,useCallback)
	if useCallback then
		local signalRes=EEPSetSignal(id,pos,useCallback)
		if signalRes>0 then return signalRes
		else return EEPSetSwitch(id,pos,useCallback) end
	else
		local signalRes=EEPSetSignal(id,pos)
		if signalRes>0 then return signalRes
		else return EEPSetSwitch(id,pos) end
	end
end

local TaskMemorizer = {
	funcs = {},
	version={0,2},
}

function TaskMemorizer:new(o)
	local tasks = {}
	setmetatable(tasks, self.funcs)
	self.funcs.__index = self.funcs
	local tasknames = {} -- store names in list, for easier comparision to excludes
	-- copy valid tasks from o to tasks
	for _,task in ipairs(o) do
		if task.signalID == nil then
			print("Task ",task.name," hat keine Signal-ID angegeben und wurde daher aus der Task-Liste entfernt")
		elseif type(task.func)~="function" then
			print("Task ",task.name," hat keine gltige Funktion angegeben und wurde daher aus der Task-Liste entfernt")
		else 
			task.interval = task.interval or 60
			task.offset = (task.offset or 0) % task.interval
			task.signalPosDone = (task.signalPosDone or (task.isCounter and 1) or 2)
			task.name=task.name or ""
			tasknames[#tasknames+1]=task.name
			task.exclude=task.exclude or {}
			tasks[#tasks+1]=task
		end
	end
	-- check, if all tasks given in the exlcude tables really exist:
	for _,task in ipairs(tasks) do
		for _,excludename in ipairs(task.exclude) do
			local found=false
			for _,taskname in ipairs(tasknames) do
				if excludename == taskname then found=true; break end
			end
			if not found then
				print("Task \"",task.name,"\": Den auszuschlieenden Task \"",excludename,"\" gibt es nicht. Vielleicht ein Tippfehler?")
			end
		end
	end
	return tasks
end

function TaskMemorizer.funcs:exec()
	local tasks=self
	local sec=EEPTime -- EEPTime may change during this function, sec does not
	tasks.lastTime=tasks.lastTime or sec-1
	if tasks.lastTime > sec then tasks.lastTime = tasks.lastTime - 86400 end -- Zeitsprung um Mitternacht (60*60*24=86400 Sekunden)
	local excludes = {} -- all excluded tasks are collected in this table (only for this single call of exec())
	for s=tasks.lastTime+1,sec do
		for _,task in ipairs(tasks) do
			if (s % task.interval) == task.offset and EEPGetElem(task.signalID) ~= task.signalPosDone then
				-- check if this task isn't in the exclude list
				local skipThisTask=false
				for _,exclude in ipairs(excludes) do
					if task.name==exclude then skipThisTask=true;print("skipped ",task.name);break end
				end
				if not skipThisTask and task.func(task) ~= false then -- Task als erledigt markieren, wenn die Funktion nicht ausdrcklich "false" zurckgibt
					if task.isCounter  then
						EEPSetElem(task.signalID,EEPGetElem(task.signalID)-1)
					else
						EEPSetElem(task.signalID,task.signalPosDone)
					end
					-- append excludes of this task to the exclude list
					for _,excludename in ipairs(task.exclude) do
						excludes[#excludes+1]=excludename
					end
				end
			end
		end
	end
	tasks.lastTime=sec
end

function TaskMemorizer.funcs:print()
	local tasks=self
	local sec=EEPTime
	print()
	print(#tasks," registrierte Aufgaben: (aktuelle Sekundenzahl: "..sec..")")
	for _,task in ipairs(tasks) do
		local status=""
		if task.isCounter then status=status..EEPGetElem(task.signalID)
		elseif EEPGetElem(task.signalID) ~= task.signalPosDone then status=status.."X"
		else status=status.."   " end
		status = status .. " | "..task.name.." (ID: "..task.signalID..", Pos: "..task.signalPosDone..", Abfrage alle "..task.interval.." Sekunden, nchste in "..((-sec+task.offset)%task.interval).." Sekunden)"
		print(status)
	end
	print()
end

print("TaskMemorizer_BH2 v",table.concat(TaskMemorizer.version,'.')," wurde eingebunden")

return TaskMemorizer
